@@ -1,16 +1,15 @@
 """URIMetadata and helper functions for metadata
 """
 import warnings
-from binascii import hexlify
 from base64 import b64decode
+from binascii import hexlify
 from collections import namedtuple
 from datetime import datetime, timezone
+
 from dateparser import parse as dateparser_parse
 from dateutil.parser import parse as dateutil_parse
-from dateutil.tz import tzlocal, tzutc
 
-
-URIMetadata = namedtuple('URIMetadata', ('exists', 'mtime', 'size', 'md5'))
+URIMetadata = namedtuple("URIMetadata", ("exists", "mtime", "size", "md5"))
 
 
 def get_seconds_from_epoch(timestamp: str) -> float:
@@ -23,9 +22,9 @@ def get_seconds_from_epoch(timestamp: str) -> float:
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
             utc_t = dateutil_parse(timestamp)
-    except:
+    except Exception:
         pass
-    if utc_t is None or utc_t.tzname() not in ('UTC', 'Z'):
+    if utc_t is None or utc_t.tzname() not in ("UTC", "Z"):
         utc_t = dateparser_parse(timestamp)
     utc_t = utc_t.astimezone(timezone.utc)
     return (utc_t - utc_epoch).total_seconds()
@@ -38,11 +37,11 @@ def base64_to_hex(b: str) -> str:
 def parse_md5_str(raw: str) -> str:
     """Check if it's based on base64 then convert it to hexadecimal string.
     """
-    raw = raw.strip('"\'')
+    raw = raw.strip("\"'")
     if len(raw) == 32:
         return raw
     else:
         try:
             return base64_to_hex(raw)
-        except:
-            return None
+        except Exception:
+            pass
